==> free-sa-1.2.4.tar.gz: free-sa-1.2.4/src/logformats/l_cgp_native.c <==
#include "global.h"

/* UNOPTIMIZED */

#define F_CGPREADRECORD "CGPReadRecordI"
struct MailRecord *CGPReadRecordI(FILE *iLog) {
    char module[5], tTime[9];
    struct MailRecord *t, tMR;
    
    
    /* read next string from log */
    while (fscanf(iLog, "%8s%*7c%4s", tTime, module) == 2) {
	t = NULL;
	
	/* SMTPI */
	if (*(const int *)module == *(const int *)((const char *)"MTPI")) {
	    if (fscanf(iLog, "%*[^(]%*c%c", &tMR.ip[0]) == 1) {
		if (tMR.ip[0] == '[') fscanf(iLog, "%[^]]%*3c", tMR.ip);
		else {
		    fscanf(iLog, "%*[^)]%*2c");
		    strcpy(tMR.ip, "127.0.0.1");
		}
		
		if (fscanf(iLog, "[%u] receive%*[^,]%*2c%lf bytes", &tMR.msgid, &tMR.bytes) == 2) {
		    t = FindNewMR(tMR.msgid);
		    
		    strcpy(t->ip, tMR.ip);
		    t->bytes = tMR.bytes;
		    t->status = 1;
		}
	    }
	} else
	
	/* QUEUE */
	if (*(const int *)module == *(const int *)((const char *)"UEUE")) {
	    tMR.name[0] = '-';
	    if (fscanf(iLog, "([%u]) from <%127[^>]", &tMR.msgid, tMR.name) >= 1) {
		t = FindNewMR(tMR.msgid);
		
		if (t->status == 1) {
		    if (tMR.name[0] == '-') t->name[0] = '-';
		    else strcpy(t->name, tMR.name);
		    t->status = 2;
		}
	    }
	} else
	
	/* DEQUEUER */
	if (*(const int *)module == *(const int *)((const char *)"EQUE")) {
	    if (fscanf(iLog, "UER [%u] %4s", &tMR.msgid, module) == 2) {
		if ((t = FindDupMR(tMR.msgid)) != NULL) {
		
		    if (t->status == 2) {
			if ((*(const int *)module == *(const int *)((const char *)"SMTP") && fscanf(iLog, "(%*[^)]%*c%s %c", t->url, &t->stat) == 2) ||
			    (*(const int *)module == *(const int *)((const char *)"LOCA") && fscanf(iLog, "L(%[^)]%*c %c", t->url, &t->stat) == 2) ||
			    (*(const int *)module == *(const int *)((const char *)"LIST") && fscanf(iLog, "(%[^)]%*cgroup %c", t->url, &t->stat) == 2)) {
			
			    switch (t->stat) {
				case 'd': 			/* delivered (LOCAL) */
				case 'r': t->code = 250; break;	/* relayed (SMTP) */ 
				case 'f': t->code = 550; break;	/* failed */
				default: SAwarning(F_CGPREADRECORD, "Unknown delivery status: %c, please contact developers", t->stat); break;
			    }
			
			    if (cfg.Site != NULL && module[0] == 'L') strcat(t->url, cfg.Site);
			    t->time = 1079005122L;
			    t->stat = 0;
			    t->count = 1;
			    t->status = 3;
			} else SAwarning(F_CGPREADRECORD, "Unknown delivery type: %c, please contact developers", module[0]);
		    }
		}
	    }
	}

	CommonReadAlign(iLog);
	
	if (t != NULL) {
	    if (t->status == 3) return t;
	    else if (t->status == -1) DelMR(t);
	}
    }

    return NULL;
}

==> free-sa-1.4.3.tar.gz: free-sa-1.4.3/src/logformats/l_cgp_native.c <==
#include "global.h"


/* UNOPTIMIZED, NOT FINISHED */
struct MailRecord *CGPReadRecordI(FILE *iLog) {
    char module[5], tTime[9];
    struct MailRecord *t, tMR;
    
    
    /* read next string from log */
    while (fscanf(iLog, "%8s%*7c%4s", tTime, module) == 2) {
	t = NULL;
	
	/* SMTPI */
	if (*(const int *)module == *(const int *)((const char *)"MTPI")) {
	    if (fscanf(iLog, "%*[^(]%*c%c", &tMR.ip[0]) == 1) {
		if (tMR.ip[0] == '[') fscanf(iLog, "%" IPLEN "[^]]%*3c", tMR.ip);
		else {
		    fscanf(iLog, "%*[^)]%*2c");
		    strcpy(tMR.ip, "127.0.0.1");
		}
		
		if (fscanf(iLog, "[%u] receive%*[^,]%*2c%lf bytes", &tMR.msgid, &tMR.bytes) == 2) {
		    t = FindNewMR(tMR.msgid);
		    
		    strcpy(t->ip, tMR.ip);
		    t->bytes = tMR.bytes;
		    t->status = 1;
		}
	    }
	} else
	
	/* QUEUE */
	if (*(const int *)module == *(const int *)((const char *)"UEUE")) {
	    tMR.name[0] = '-';
	    if (fscanf(iLog, "([%u]) from <%" USERNAMELEN "[^>]", &tMR.msgid, tMR.name) >= 1) {
		t = FindNewMR(tMR.msgid);
		
		if (t->status == 1) {
		    if (tMR.name[0] == '-') t->name[0] = '-';
		    else strcpy(t->name, tMR.name);
		    t->status = 2;
		}
	    }
	} else
	
	/* DEQUEUER */
	if (*(const int *)module == *(const int *)((const char *)"EQUE")) {
	    if (fscanf(iLog, "UER [%u] %4s", &tMR.msgid, module) == 2) {
		if ((t = FindDupMR(tMR.msgid)) != NULL) {
		
		    if (t->status == 2) {
			if ((*(const int *)module == *(const int *)((const char *)"SMTP") && fscanf(iLog, "(%*[^)]%*c%" URLLEN "s %c", t->url, &t->stat) == 2) ||
			    (*(const int *)module == *(const int *)((const char *)"LOCA") && fscanf(iLog, "L(%" URLLEN "[^)]%*c %c", t->url, &t->stat) == 2) ||
			    (*(const int *)module == *(const int *)((const char *)"LIST") && fscanf(iLog, "(%" URLLEN "[^)]%*cgroup %c", t->url, &t->stat) == 2)) {
			
			    switch (t->stat) {
				case 'd': 			/* delivered (LOCAL) */
				case 'r': t->code = 250; break;	/* relayed (SMTP) */ 
				case 'f': t->code = 550; break;	/* failed */
				default: SAwarning("Unknown delivery status: %c, please contact developers", t->stat); break;
			    }
			
			    if (cfg.Site != NULL && module[0] == 'L') strcat(t->url, cfg.Site);
			    t->time = 1079005122L;
			    t->stat = 0;
			    t->count = 1;
			    t->status = 3;
			} else SAwarning("Unknown delivery type: %c, please contact developers", module[0]);
		    }
		}
	    }
	}

	CommonReadAlign(iLog);
	
	if (t != NULL) {
	    if (t->status == 3) return t;
	    else if (t->status == -1) DelMR(t);
	}
    }

    return NULL;
}
